# -*- coding: utf-8 -*-
##############################################################################
#
#    Author: Guewen Baconnier
#    Copyright 2012 Camptocamp SA
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import xmlrpclib

from osv import osv, fields
from tools.translate import _


class sale_order(osv.osv):

    _inherit = 'sale.order'

    _columns = {'magento_ref': fields.char('Magento Invoice ID', size=32),
                'allow_magento_manual_invoice': fields.related(
                'base_payment_type_id', 'allow_magento_manual_invoice',
                type='boolean',
                string="Allow Manual Creation of Magento Invoice")}

    def _magento_create_invoice(self, cr, uid, order, context=None):
        shop = order.shop_id

        ctx = context.copy()
        ctx.update({
            'shop_id': shop.id,
            'external_referential_type':
                shop.referential_id.type_id.name, })

        referential = shop.referential_id
        connection = referential.external_connection()
        try:
            external_invoice = self._create_external_invoice(
                cr, uid, order, connection, referential.id, context=ctx)
        except xmlrpclib.Fault, magento_error:
            # TODO: in case of error on Magento because the invoice has
            # already been created, get the invoice number
            # and store it in magento_ref
            raise osv.except_osv(
                _('Error'), _("Error on Magento on the invoice creation "
                              "for order %s :\n" \
                              "%s" % (order.name, magento_error)))
        self.write(
            cr, uid, order.id,
            {'magento_ref': external_invoice},
            context=context)
        cr.commit()

        self._check_need_to_update_single(
            cr, uid, order, connection, context=ctx)
        return True

    def button_magento_create_invoice(self, cr, uid, ids, context=None):
        order = self.browse(cr, uid, ids[0], context=context)
        if order.state != 'draft':
            raise osv.except_osv(
                _('Error'), _('This order is not a quotation.'))

        if not order.is_magento:
            raise osv.except_osv(
                _('Error'), _('This is not a Magento sale order.'))

        if not order.base_payment_type_id:
            raise osv.except_osv(
                _('Error'), _('This order has no external '
                              'payment type settings.'))

        if not order.base_payment_type_id.allow_magento_manual_invoice:
            raise osv.except_osv(
                _('Error'), _(
                    "Manual creation of the invoice on Magento "
                    "is forbidden for external payment : %s" %
                    order.ext_payment_method))

        # sale_exceptions module methods
        # in order to check if the order is valid
        # before create the invoice on magento
        # it maybe has something to correct
        exception_ids = self.detect_exceptions(
            cr, uid, [order.id], context=context)
        if exception_ids:
            return self._popup_exceptions(
                cr, uid, order.id,  context=context)

        self._magento_create_invoice(cr, uid, order, context=context)
        return True

    def _call_magento_invoice(self, cr, uid, order, order_external_data=None, context=None):
        """Return the Magento data for all invoices
        of an order

        :param browse_record order: browsable record of sale order
        :param dict order_external_data: magento data of the order, optional,
        a call to magento will be done to get the data if None
        :return: list of magento invoices as dict
        """
        if context is None:
            context = {}

        referential = order.shop_id.referential_id
        connection = context.get('conn_obj') or \
            referential.external_connection()

        # Filter on order_increment_id does not work
        # on magento.
        # So we need first to get the order id
        # and filter the invoice on this order id
        if not order_external_data:
            order_ext_id = self.oeid_to_extid(
                cr, uid, order.id, referential.id, context=context)
            order_external_data = connection.call(
                'sales_order.info', [order_ext_id])

        mag_order_id = order_external_data['order_id']
        return connection.call(
            'sales_order_invoice.list', [{'order_id': {'eq': mag_order_id}}])

    def _get_missing_magento_invoice_number(self, cr, uid, order, external_data, context=None):
        """Get the first invoice number for a magento order
        and assign it to magento_ref fields
        so we can later push it on the invoice

        :param browse_record order: browsable record of sale order
        :param dict external_data: magento data of the order
        :return: invoice number of magento
        """
        invoice_number = False
        mag_invoices = self._call_magento_invoice(
            cr, uid, order, order_external_data=external_data, context=context)
        if mag_invoices:
            invoice_number = mag_invoices[0].get('increment_id')

        if invoice_number:
            self.write(
                cr, uid, order.id,
                {'magento_ref': invoice_number},
                context=context)
            # propagate on invoice if some exists
            # don't care of multiplue invoice because we
            # don't know which invoice would have
            # which magento number, 1 invoice on magento
            # and 1 invoice on openerp expected, otherwise
            # it has to be manually filled
            if order.invoice_ids:
                inv_ids = [inv.id for inv in order.invoice_ids]
                self.pool.get('account.invoice').write(
                    cr, uid, inv_ids,
                    {'magento_ref': invoice_number},
                    context=context)
        return invoice_number

    def create_payments(self, cr, uid, order_id, data_record, context=None):
        """Inherited to get the magento invoice number.
        With Magento, we are sure that when we have a payment,
        we have an invoice, so we get its number to have it
        on the openerp invoice
        """
        paid = super(sale_order, self).create_payments(
            cr, uid, order_id, data_record, context=context)

        if 'Magento' in context.get('external_referential_type', ''):
            order = self.browse(cr, uid, order_id, context=context)

            if paid and not order.magento_ref:
                # get the invoice number
                self._get_missing_magento_invoice_number(
                    cr, uid, order, data_record, context=context)

        return paid

    def _prepare_invoice(self, cr, uid, order, lines, context=None):
        """Prepare the dict of values to create the new invoice for a
           sale order. This method may be overridden to implement custom
           invoice generation (making sure to call super() to establish
           a clean extension chain).

           Inherited in order to:
           Update the Magento invoice id because the invoice has been
           created from the sale order and it is stored on the sale order
           first.

           :param browse_record order: sale.order record to invoice
           :param list(int) lines: list of invoice line IDs that must be
                                   attached to the invoice
           :return: dict of value to create() the invoice
        """
        vals = super(sale_order, self)._prepare_invoice(
            cr, uid, order, lines, context=context)
        if order.magento_ref:
            vals['magento_ref'] = order.magento_ref
        return vals

    def _apply_payment_settings(self, cr, uid, order, paid, context=None):
        """We should deactivate need_to_update to avoid to query magento
        on each import of orders and not to mislead the users if they use
        a filter on need_to_update.
        """
        res = super(sale_order, self)._apply_payment_settings(cr, uid, order,
            paid, context=context)
        payment_settings = order.base_payment_type_id
        if payment_settings and payment_settings.allow_magento_manual_invoice:
            self.write(cr, uid, order.id, {'need_to_update': False}, context=context)
        return res

sale_order()
